library(tweeDEseq)
library(DESeq)
library(edgeR)
library(parallel)
library(RColorBrewer)

## number of processors to use when calling tweeDEseq
nCores <- 16

## parameters controlling the simulation
p <- 20000                                 ## total number of genes
pDE <- 2000                                ## number of DE genes
fc <- 1.5                                  ## fold-change
n <- 70                                    ## sample size
grp <- rep(1:2, each=n/2)                  ## sample group indicator variable
nsims <- 100                               ## number of simulations
sdoffset <- 0.5                            ## SD offset library size
sim.name <- sprintf("qvalXeFDR.p%d.pDE%d.n%d.fc%.1f.sdoff%.1f", p, pDE, n, fc, sdoffset) ## name identifying the previous parameters to store resulting files

##
## data simulation procedure adapted from the example of
## QL.fit() in the QuasiSeq package which simulated count
## data using a gamma-Poisson model
##

simCountData <- function(p=1000, n=40, cond=rep(1:2, each=n/2), pDE=100,
                         fc=2, fUp=0.5, sd.offset=0) {

  ## Simulate means for p genes identical in the two sample groups
  mu <- matrix(exp(rnorm(p, mean=2.5, sd=2)), nrow=p, ncol=2)

  upDEgenes <- 1:floor(pDE*fUp)
  dnDEgenes <- (floor(pDE*fUp)+1):pDE

  ## Make first pDE genes DE according to the fold-change 'fc' and
  ## the fraction of up-regulated genes 'fUp'
  mu[upDEgenes, 1] <- mu[upDEgenes, 1] * fc
  mu[dnDEgenes, 1] <- mu[dnDEgenes, 1] / fc

  ## Simulate dispersion parameters
  phi <- rgamma(p, 2, .7) + 1

  b <- 1 / (phi-1)
  a <- mu[, cond] * b

  ## Simulate library size factors
  if (sd.offset > 0) {
    offset <- 2^(rnorm(n, 0, sd.offset))
    a <- t(t(a) * offset)
  }

  ## Sample the Poisson lambda parameters
  lambda <- matrix(rgamma(p*n, a,rep(b, n)), nrow=p, ncol=n)

  ## Sample count data
  Y <- matrix(rpois(p*n, lambda), nrow=p, ncol=n)

  Y
}

options(warn=-1) ## for the DESeq warning about sharingMode="gene-est-only"

simData <- list()
pv <- list(tweeDEseq=vector(mode="list", length=nsims),
           DESeqPgOnly=vector(mode="list", length=nsims),
           DESeqPmax=vector(mode="list", length=nsims),
           DESeqCgOnly=vector(mode="list", length=nsims),
           DESeqCmax=vector(mode="list", length=nsims),
           edgeRdf20=vector(mode="list", length=nsims),
           edgeRdf1=vector(mode="list", length=nsims),
           edgeRqlfdf20=vector(mode="list", length=nsims),
           edgeRqlfdf1=vector(mode="list", length=nsims))

cat("tweeDEseq:\n")

for (i in 1:nsims) {
  cat("i=", i, "\n")
  Y <- simCountData(p, n, grp, pDE, fc, sdoffset)
  Y <- Y[rowSums(Y) >= 10, ]
  simData[[i]] <- Y

  ## tweeDEseq
  YnormPT <- normalizeCounts(Y, grp)
  resPT <- tweeDE(YnormPT, grp, mc.cores=nCores)
  pv$tweeDEseq[[i]] <- resPT$pval
}

cat("Other methods:\n")

coreID <- mclapply(as.list(1:nCores), function(x) parallel:::masterDescriptor(), mc.cores=nCores)[[1]]
pb <- txtProgressBar(min=0, max=length(simData), style=3)
i <- 0
pvOther <- mclapply(simData, function(Y) {
  if (parallel:::masterDescriptor() == coreID) {
    i <<- i + 1
    setTxtProgressBar(pb, nCores*i)
  }

  ## DESeq (fitType="local" is here necessary since "parametric" gives an error with these data
  ## Error in parametricDispersionFit(means, disps) : Parametric dispersion fit failed. Try a local fit and/or a pooled estimation.
  cds <- newCountDataSet(Y, conditions=grp)
  ## sizeFactors(cds) <- rep(1, n)
  cds <- estimateSizeFactors(cds)

  cdsPgOn <- estimateDispersions(cds, method="pooled", fitType="local", sharingMode="gene-est-only")
  resDESeq <- nbinomTest(cdsPgOn, "1", "2")
  DESeqPgOnly <- resDESeq$pval

  cdsPmax <- estimateDispersions(cds, method="pooled", fitType="local", sharingMode="maximum")
  resDESeq <- nbinomTest(cdsPmax, "1", "2")
  DESeqPmax <- resDESeq$pval

  cdsCgOn <- estimateDispersions(cds, method="per-condition", fitType="local", sharingMode="gene-est-only")
  resDESeq <- nbinomTest(cdsCgOn, "1", "2")
  DESeqCgOnly <- resDESeq$pval

  cdsCmax <- estimateDispersions(cds, method="per-condition", fitType="local", sharingMode="maximum")
  resDESeq <- nbinomTest(cdsCmax, "1", "2")
  DESeqCmax <- resDESeq$pval

  ## edgeR
  d <- DGEList(counts=Y, group=grp, lib.size=colSums(Y))
  d <- calcNormFactors(d)
  design <- model.matrix( ~ d$sample$group)
  d.common <- estimateGLMCommonDisp(d, design)
  d.trended <- estimateGLMTrendedDisp(d.common, design)
  d.tagwise <- estimateGLMTagwiseDisp(d.trended, design)
  d.tagwise.df1 <- estimateGLMTagwiseDisp(d.trended, design, prior.df=1)
  d.fit <- glmFit(d.tagwise, design)
  d.fit.df1 <- glmFit(d.tagwise.df1, design)
  resEdgeR <- glmLRT(d.fit, coef=2)
  edgeRdf20 <- resEdgeR$table$PValue
  resEdgeRdf1 <- glmLRT(d.fit.df1, coef=2)
  edgeRdf1 <- resEdgeRdf1$table$PValue
  resEdgeRqlf <- glmQLFTest(d.fit, coef=2)
  edgeRqlfdf20 <- resEdgeRqlf$table$PValue
  resEdgeRqlfdf1 <- glmQLFTest(d.fit.df1, coef=2)
  edgeRqlfdf1 <- resEdgeRqlfdf1$table$PValue

  list(DESeqPgOnly=DESeqPgOnly, DESeqPmax=DESeqPmax, DESeqCgOnly=DESeqCgOnly, DESeqCmax=DESeqCmax,
       edgeRdf20=edgeRdf20, edgeRdf1=edgeRdf1, edgeRqlfdf20=edgeRqlfdf20, edgeRqlfdf1=edgeRqlfdf1)
}, mc.cores=nCores)
close(pb)

for (method in names(pv[-1])) {
  pv[[method]] <- lapply(pvOther, function(x, method) x[[method]], method)
}

## save resulting data
save(pv, simData, p, pDE, n, grp, nsims, file=paste0(sim.name, ".RData"))

## plot resulting figures

library(qvalue)

eFDR <- function(pval, qval, de.genes, qval.cutoffs) {

  y <- sapply(qval.cutoffs, function(q, pval, qval, de.genes) {
                efdr <- 0
                ## de.calls <- names(pval)[pval <= pval[max(c(0,which(qval <= q)))]]
                if (any(qval <= q)) {
                  de.calls <- which(pval <= max(pval[which(qval <= q)]))
                  if (length(de.calls) > 0)
                    efdr <- sum(is.na(match(de.calls, de.genes))) / length(de.calls)
                }
                efdr
              }, pval, qval, de.genes)

  cbind(qval.cutoffs, y)
}

qval.cutoffs <- seq(0, 0.1, by=0.001)
## qval.cutoffs <- seq(0, 1, by=0.01)
all.eFDR <- list(tweeDEseq=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 DESeqPgOnly=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 DESeqPmax=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 DESeqCgOnly=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 DESeqCmax=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 edgeRdf20=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 edgeRqlfdf20=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 edgeRdf1=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims),
                 edgeRqlfdf1=matrix(NA_real_, nrow=length(qval.cutoffs), ncol=nsims))
all.pi0 <- list(tweeDEseq=rep(NA_real_, nsims),
                DESeqPgOnly=rep(NA_real_, nsims),
                DESeqPmax=rep(NA_real_, nsims),
                DESeqCgOnly=rep(NA_real_, nsims),
                DESeqCmax=rep(NA_real_, nsims),
                edgeRdf20=rep(NA_real_, nsims),
                edgeRqlfdf20=rep(NA_real_, nsims),
                edgeRdf1=rep(NA_real_, nsims),
                edgeRqlfdf1=rep(NA_real_, nsims))

for (i in 1:nsims) {
  pval <- pv$tweeDEseq[[i]]
  qobj <- qvalue(pval)
  all.eFDR$tweeDEseq[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$tweeDEseq[i] <- qobj$pi0
  pval <- pv$DESeqPgOnly[[i]]
  qobj <- qvalue(pval)
  all.eFDR$DESeqPgOnly[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$DESeqPgOnly[i] <- qobj$pi0
  pval <- pv$DESeqPmax[[i]]
  qobj <- qvalue(pval)
  all.eFDR$DESeqPmax[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$DESeqPmax[i] <- qobj$pi0
  pval <- pv$DESeqCgOnly[[i]]
  qobj <- qvalue(pval)
  all.eFDR$DESeqCgOnly[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$DESeqCgOnly[i] <- qobj$pi0
  pval <- pv$DESeqCmax[[i]]
  qobj <- qvalue(pval)
  all.eFDR$DESeqCmax[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$DESeqCmax[i] <- qobj$pi0
  pval <- pv$edgeRdf20[[i]]
  qobj <- qvalue(pval)
  all.eFDR$edgeRdf20[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$edgeRdf20[i] <- qobj$pi0
  pval <- pv$edgeRqlfdf20[[i]]
  qobj <- qvalue(pval)
  all.eFDR$edgeRqlfdf20[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$edgeRqlfdf20[i] <- qobj$pi0
  pval <- pv$edgeRdf1[[i]]
  qobj <- qvalue(pval)
  all.eFDR$edgeRdf1[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$edgeRdf1[i] <- qobj$pi0
  pval <- pv$edgeRqlfdf1[[i]]
  qobj <- qvalue(pval)
  all.eFDR$edgeRqlfdf1[, i] <- eFDR(pval, qobj$qvalues, 1:pDE, qval.cutoffs)[, 2]
  all.pi0$edgeRqlfdf1[i] <- qobj$pi0
}

pp <- colorRampPalette(brewer.pal(12, "Paired"))(12)
collist <- c(pp[6], pp[2], pp[1], pp[10], pp[9], pp[5], pp[7], pp[8], pp[12])

pdf(file=paste0(sim.name, ".pdf"), height=7, width=7)
par(mar=c(4, 4, 1, 1))
plot(qval.cutoffs, rowMeans(all.eFDR$tweeDEseq), type="l", lwd=3,
     col=collist[1], xlab="q-values", ylab="eFDR", ## ylim=c(0, 1)) ## , xlim=c(0, 1))
     ylim=range(sapply(all.eFDR, rowMeans), na.rm=TRUE))
for (i in 2:length(all.eFDR)) {
  lines(qval.cutoffs, rowMeans(all.eFDR[[i]]), type="l", lwd=3, col=collist[i])
}
abline(0, 1, lwd=2, lty=2)
legend("topleft", names(all.eFDR), col=collist, lwd=3, lty=1, inset=0.01)
dev.off()

pdf(file=paste0(sim.name, ".estNrDEgenes.pdf"), height=5, width=12)
par(mar=c(5, 4, 1, 1))
boxplot(lapply(all.pi0, function(x, p) p*(1-x), p), names=rep("", 9), ylab="Estimated # of DE genes", col=collist, las=1, cex.lab=1.2)
points(1:length(all.pi0)+0.1, sapply(all.pi0, function(x, p) mean(p*(1-x)), p), pch=23, bg="black")
abline(h=pDE, lwd=2, lty=2)
par(xpd=TRUE)
text(1:9, par("usr")[3]*4, names(all.pi0), srt=45, cex=1.2, pos=1)
dev.off()
