library(tweeDEseq)
library(edgeR)
library(DESeq)
library(limma)
library(parallel)
library(Biobase)
library(tweeDEseqCountData)
library(cqn)

## Number of cpu cores for calculating in parallel

nCores <- 16

## Number of simulations
nsims <- 100

## Load data

data(pickrell1)
countsRaw <- exprs(pickrell1.eset)
initialLibSizes <- colSums(countsRaw)

## Filter counts by discarding genes with low expression
countsFiltered <- filterCounts(countsRaw, mean.cpm.cutoff=0.5)
dim(countsFiltered)

## Filter counts by discarding genes without length of G+C content annotation
data(annotEnsembl63)

common <- intersect(rownames(countsFiltered),
                    rownames(annotEnsembl63[!is.na(annotEnsembl63$Length), ]))
countsFiltered <- countsFiltered[common, ]

## Normalize counts with cqn
cqnNorm <- cqn(countsFiltered,
               lengths=as.numeric(annotEnsembl63[common, "Length"]),
               x=annotEnsembl63[common, "GCcontent"],
               sizeFactors=initialLibSizes, verbose=FALSE)
log2rpm <- cqnNorm$y + cqnNorm$offset
log2r <- sweep(cqnNorm$y + cqnNorm$offset, 2, log2(initialLibSizes/1e6), FUN="+")
countsNormCQN <- ceiling(2^log2r-0.5)
dim(countsNormCQN)

## Restrict to Men vs Men samples
males <- which(pickrell1.eset$gender == "male")
countsFilteredMales <- countsFiltered[, males]
countsNormCQNmales <- countsNormCQN[, males]
cqnNormMales <- cqnNorm
cqnNormMales$offset <- cqnNormMales$offset[, males]

## get bootstrappedMvsM from *Un simulation
load("bootstrappedMvsM.RData")

## do first tweeDEseq parallelizing calculations in each simulation
for(i in 1:nsims){
  cat("i=", i, "\n")

  ## get bootstrappedMvsM from the *Un simulation
  countsNormCQNmales <- countsNormCQN[, bootstrappedMvsM[[i]]]
  cqnNormMales <- cqnNorm
  cqnNormMales$offset <- cqnNormMales$offset[, bootstrappedMvsM[[i]]]


  ## tweeDEseq
  resPT[[i]] <- tweeDE(countsNormCQNmales,
                       group = c(rep("grp1", 20), rep("grp2", 20)),
                       mc.cores = nCores)
}

## save intermediate results
save(resPT, bootstrappedMvsM, countsRaw, initialLibSizes, males, countsFiltered,
     countsNormCQN, cqnNorm, file = "MvsM_bootstrappingCQN.RData")

## do second edgeR and DESeq distributing simulations in parallel using the SAME bootstrapped groups

colnames(countsNormCQN) <- colnames(countsFiltered) <- NULL

coreID <- mclapply(as.list(1:nCores), function(x) parallel:::masterDescriptor(), mc.cores=nCores)[[1]]
pb <- txtProgressBar(min=0, max=length(bootstrappedMvsM), style=3)
i <- 0
res.other <- mclapply(bootstrappedMvsM, function(grp) {
  if (parallel:::masterDescriptor() == coreID) {
    i <<- i + 1
    setTxtProgressBar(pb, nCores*i)
  }
  countsFilteredMales <- countsFiltered[, grp] ## for bootstrap with replacement
  countsNormCQNmales <- countsNormCQN[, grp] ## for bootstrap with replacement

  ## edgeR - common/trended/tagwise - prior.df 20 (default) & 1 - LRT/QLF strategies
  d <- DGEList(counts=countsFilteredMales, group=c(rep("grp1", 20), rep("grp2", 20)), lib.size = initialLibSizes[grp])
  design <- model.matrix(~ d$sample$group)
  d.common <- estimateGLMCommonDisp(d, design, offset=cqnNorm$offset[, grp])
  d.trended <- estimateGLMTrendedDisp(d.common, design, offset=cqnNorm$offset[, grp])
  d.tagwise <- estimateGLMTagwiseDisp(d.trended, design, offset=cqnNorm$offset[, grp])
  d.tagwise.df1 <- estimateGLMTagwiseDisp(d.trended, design, prior.df=1, offset=cqnNorm$offset[, grp])
  d.fit <- glmFit(d.tagwise, design, offset=cqnNorm$offset[, grp])
  d.fit.df1 <- glmFit(d.tagwise.df1, design, offset=cqnNorm$offset[, grp])
  resEdgeR <- glmLRT(d.fit, coef=2)
  resEdgeR.df1 <- glmLRT(d.fit.df1, coef=2)
  resEdgeR.qlf <- glmQLFTest(d.fit, coef=2)
  resEdgeR.qlf.df1 <- glmQLFTest(d.fit.df1, coef=2)

  ## DESeq - pooled gene-est-only
  cdsNormCQN <- newCountDataSet(countsNormCQNmales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsNormCQN) <- rep(1, ncol(countsNormCQNmales))
  cdsNormCQN <- estimateDispersions(cdsNormCQN, method = "pooled",
                                    sharingMode = "gene-est-only")
  resDESeqPgOnly <- nbinomTest(cdsNormCQN, "grp1", "grp2")

  ## DESeq - pooled maximum
  cdsNormCQN <- newCountDataSet(countsNormCQNmales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsNormCQN) <- rep(1, ncol(countsNormCQNmales))
  cdsNormCQN <- estimateDispersions(cdsNormCQN, method = "pooled",
                                    sharingMode = "maximum")
  resDESeqPmax <- nbinomTest(cdsNormCQN, "grp1", "grp2")

  ## DESeq - per-condition gene-est-only
  cdsNormCQN <- newCountDataSet(countsNormCQNmales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsNormCQN) <- rep(1, ncol(countsNormCQNmales))
  cdsNormCQN <- estimateDispersions(cdsNormCQN, method = "per-condition",
                                    sharingMode = "gene-est-only")
  resDESeqCgOnly <- nbinomTest(cdsNormCQN, "grp1", "grp2")

  ## DESeq - per-condition maximum
  cdsNormCQN <- newCountDataSet(countsNormCQNmales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsNormCQN) <- rep(1, ncol(countsNormCQNmales))
  cdsNormCQN <- estimateDispersions(cdsNormCQN, method = "per-condition",
                                    sharingMode = "maximum")
  resDESeqCmax <- nbinomTest(cdsNormCQN, "grp1", "grp2")

  ## limma on log2 CPM
  design <- model.matrix(~ factor(c(rep("grp1", 20), rep("grp2", 20))))
  log2cpmNormCQNmales <- predFC(countsNormCQNmales, design=NULL) ## use edgeR::predFC() to obtain log2 CPM
  fit <- lmFit(log2cpmNormCQNmales, design)
  fit <- eBayes(fit)
  resLimma <- topTable(fit, coef=2, n=Inf)

  list(edgeR=resEdgeR, edgeRdf1=resEdgeR.df1, edgeRqlf=resEdgeR.qlf, edgeRqlfdf1=resEdgeR.qlf.df1,
       DESeqPgOnly=resDESeqPgOnly, DESeqPmax=resDESeqPmax, DESeqCgOnly=resDESeqCgOnly, DESeqCmax=resDESeqCmax)
}, mc.cores=nCores)
close(pb)

## put all results together in a single list
res <- list(tweeDEseq=resPT)
res[["edgeR"]] <- lapply(res.other, function(x) x$edgeR)
res[["edgeRdf1"]] <- lapply(res.other, function(x) x$edgeRdf1)
res[["edgeRqlf"]] <- lapply(res.other, function(x) x$edgeRqlf)
res[["edgeRqlfdf1"]] <- lapply(res.other, function(x) x$edgeRqlfdf1)
res[["DESeqPgOnly"]] <- lapply(res.other, function(x) x$DESeqPgOnly)
res[["DESeqPmax"]] <- lapply(res.other, function(x) x$DESeqPmax)
res[["DESeqCgOnly"]] <- lapply(res.other, function(x) x$DESeqCgOnly)
res[["DESeqCmax"]] <- lapply(res.other, function(x) x$DESeqCmax)

save(res, bootstrappedMvsM, countsRaw, initialLibSizes, males, countsFiltered,
     countsNormCQN, cqnNorm, file = "MvsM_bootstrappingCQN.RData")

sessionInfo()
