library(tweeDEseq)
library(edgeR)
library(DESeq)
library(limma)
library(parallel)
library(Biobase)
library(tweeDEseqCountData)

## Number of cpu cores for calculating in parallel

nCores <- 16

## Number of simulations
nsims <- 100

## Load data

data(pickrell1)
countsRaw <- exprs(pickrell1.eset)
initialLibSizes <- colSums(countsRaw)

## Filter counts by discarding genes with low expression
countsFiltered <- filterCounts(countsRaw, mean.cpm.cutoff=0.5)
dim(countsFiltered)

## Restrict to Women vs Women samples
males <- which(pickrell1.eset$gender == "male")
countsFilteredMales <- countsFiltered[, males]

resPT <- bootstrappedMvsM <- list()
set.seed(1234)

## do first tweeDEseq parallelizing calculations in each simulation
for(i in 1:nsims){
  cat("i=", i, "\n")

  ## bootstrap two groups with half of the samples each with replacement
  bootstrappedMvsM[[i]] <- c(sample(males, size=20, replace=TRUE), sample(males, size=20, replace=TRUE))
  countsFilteredMales <- countsFiltered[, bootstrappedMvsM[[i]]]

  ## tweeDEseq
  resPT[[i]] <- tweeDE(countsFilteredMales, group = c(rep("grp1", 20), rep("grp2", 20)),  mc.cores = nCores)
}

## save intermediate results
save(bootstrappedMvsM, file="bootstrappedMvsM.RData")
save(resPT, bootstrappedMvsM, countsRaw, initialLibSizes, males, countsFiltered,
     file = "MvsM_bootstrappingUn.RData")

## do second edgeR and DESeq distributing simulations in parallel using the SAME bootstrapped groups

colnames(countsFiltered) <- NULL

coreID <- mclapply(as.list(1:nCores), function(x) parallel:::masterDescriptor(), mc.cores=nCores)[[1]]
pb <- txtProgressBar(min=0, max=length(bootstrappedMvsM), style=3)
i <- 0
res.other <- mclapply(bootstrappedMvsM, function(grp) {
  if (parallel:::masterDescriptor() == coreID) {
    i <<- i + 1
    setTxtProgressBar(pb, nCores*i)
  }
  countsFilteredMales <- countsFiltered[, grp] ## for bootstrap with replacement

  ## edgeR - common/trended/tagwise - prior.df 20 (default) & 1 - LRT/QLF strategies
  d <- DGEList(counts=countsFilteredMales, group=c(rep("grp1", 20), rep("grp2", 20)), lib.size = initialLibSizes[grp])
  design <- model.matrix(~ d$sample$group)
  d.common <- estimateGLMCommonDisp(d, design)
  d.trended <- estimateGLMTrendedDisp(d.common, design)
  d.tagwise <- estimateGLMTagwiseDisp(d.trended, design)
  d.tagwise.df1 <- estimateGLMTagwiseDisp(d.trended, design, prior.df=1)
  d.fit <- glmFit(d.tagwise, design)
  d.fit.df1 <- glmFit(d.tagwise.df1, design)
  resEdgeR <- glmLRT(d.fit, coef=2) ## already calculated
  resEdgeR.df1 <- glmLRT(d.fit.df1, coef=2)
  resEdgeR.qlf <- glmQLFTest(d.fit, coef=2)
  resEdgeR.qlf.df1 <- glmQLFTest(d.fit.df1, coef=2)

  ## DESeq - pooled gene-est-only
  cdsUn <- newCountDataSet(countsFilteredMales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsUn) <- rep(1, ncol(countsFilteredMales))
  cdsUn <- estimateDispersions(cdsUn, method = "pooled",
                               sharingMode = "gene-est-only")
  resDESeqPgOnly <- nbinomTest(cdsUn, "grp1", "grp2")

  ## DESeq - pooled maximum
  cdsUn <- newCountDataSet(countsFilteredMales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsUn) <- rep(1, ncol(countsFilteredMales))
  cdsUn <- estimateDispersions(cdsUn, method = "pooled",
                               sharingMode = "maximum")
  resDESeqPmax <- nbinomTest(cdsUn, "grp1", "grp2")

  ## DESeq - per-condition gene-est-only
  cdsUn <- newCountDataSet(countsFilteredMales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsUn) <- rep(1, ncol(countsFilteredMales))
  cdsUn <- estimateDispersions(cdsUn, method = "per-condition",
                               sharingMode = "gene-est-only")
  resDESeqCgOnly <- nbinomTest(cdsUn, "grp1", "grp2")

  ## DESeq - per-condition maximum
  cdsUn <- newCountDataSet(countsFilteredMales, conditions=c(rep("grp1", 20), rep("grp2", 20)))
  sizeFactors(cdsUn) <- rep(1, ncol(countsFilteredMales))
  cdsUn <- estimateDispersions(cdsUn, method = "per-condition",
                               sharingMode = "maximum")
  resDESeqCmax <- nbinomTest(cdsUn, "grp1", "grp2")

  list(edgeR=resEdgeR, edgeRdf1=resEdgeR.df1, edgeRqlf=resEdgeR.qlf, edgeRqlfdf1=resEdgeR.qlf.df1,
       DESeqPgOnly=resDESeqPgOnly, DESeqPmax=resDESeqPmax, DESeqCgOnly=resDESeqCgOnly, DESeqCmax=resDESeqCmax)
}, mc.cores=nCores)
close(pb)

## put all results together in a single list
res <- list(tweeDEseq=resPT)
res[["edgeR"]] <- lapply(res.other, function(x) x$edgeR)
res[["edgeRdf1"]] <- lapply(res.other, function(x) x$edgeRdf1)
res[["edgeRqlf"]] <- lapply(res.other, function(x) x$edgeRqlf)
res[["edgeRqlfdf1"]] <- lapply(res.other, function(x) x$edgeRqlfdf1)
res[["DESeqPgOnly"]] <- lapply(res.other, function(x) x$DESeqPgOnly)
res[["DESeqPmax"]] <- lapply(res.other, function(x) x$DESeqPmax)
res[["DESeqCgOnly"]] <- lapply(res.other, function(x) x$DESeqCgOnly)
res[["DESeqCmax"]] <- lapply(res.other, function(x) x$DESeqCmax)

save(res, bootstrappedMvsM, countsRaw, initialLibSizes, males, countsFiltered,
     file = "MvsM_bootstrappingUn.RData")

sessionInfo()
